import pygame

#Game Setup
pygame.init()
screen = pygame.display.set_mode((800, 400))
pygame.display.set_caption('Demo')
clock = pygame.time.Clock()
# test_font = pygame.font.Font('font/XXXXX.ttf', 50)
game_active = True

#Load immage as surface
player_surface = pygame.image.load('data/alien3.png').convert_alpha()

#Main game loop
while True:
    for event in pygame.event.get():
        if event.type == pygame.QUIT:
            pygame.quit()
            exit()

    if game_active:
        screen.blit(player_surface, (400, 200))

    pygame.display.update()
    clock.tick(60)



import pygame
import os

#Setup game
def Main():
    # Game Setup
    pygame.init()
    screen = pygame.display.set_mode((600, 500))
    pygame.display.set_caption('Demo')
    clock = pygame.time.Clock()
    time_elapsed_since_start = 0
    delta_time = 0
    # test_font = pygame.font.Font('font/XXXXX.ttf', 50)
    game_active = True

    # Player Class
    class Player(pygame.sprite.Sprite):

        def __init__(self, startLocation):
            super().__init__()
            player_image_frame1 = pygame.image.load(os.path.join(os.path.dirname(__file__), 'data/alien3.png')).convert_alpha()
            player_image_frames = [player_image_frame1]
            self.image = player_image_frames[0]
            self.rect = self.image.get_rect(center=startLocation)
            self.speed = 4

        def player_input(self):
            keys = pygame.key.get_pressed()
            if keys[pygame.K_LEFT]:
                self.rect.x -= self.speed
            if keys[pygame.K_RIGHT]:
                self.rect.x += self.speed
            # if keys[pygame.K_UP]:
            #     self.rect.y -= self.speed
            # if keys[pygame.K_DOWN]:
            #     self.rect.y += self.speed

        def update(self):
            self.player_input()

    # Player Class
    class Enemy(pygame.sprite.Sprite):

        def __init__(self, startLocation):
            super().__init__()
            enemy_image_frame1 = pygame.image.load(os.path.join(os.path.dirname(__file__), 'data/enemy2.png')).convert_alpha()
            enemy_image_frame1 = pygame.transform.scale(enemy_image_frame1, (50, 50))
            player_image_frames = [enemy_image_frame1]
            self.image = player_image_frames[0]
            self.rect = self.image.get_rect(center=startLocation)
            self.speed = 1
            self.direction = 1
            self.move_timer = 0
            self.move_threshold = 30

        def move(self):
            self.move_timer += delta_time
            if self.direction == 1:
                if self.move_timer > self.move_threshold:
                    self.rect.x -= self.speed
                    self.move_timer = 0
            # if keys[pygame.K_UP]:
            #     self.rect.y -= self.speed
            # if keys[pygame.K_DOWN]:
            #     self.rect.y += self.speed

        def update(self):
            self.move()


    player = pygame.sprite.GroupSingle()
    player.add(Player(startLocation=(300, 450)))

    enemies = pygame.sprite.Group()
    for i in range(50, 151, 50):
        for j in range(80, 520, 60):
            enemies.add(Enemy(startLocation=(j, i)))

    #Main game loop
    while True:
        screen.fill((0, 0, 0))

        for event in pygame.event.get():
            if event.type == pygame.QUIT:
                pygame.quit()
                exit()

        if game_active:
            player.draw(screen)
            player.update()
            enemies.draw((screen))
            enemies.update()


        pygame.display.update()
        delta_time = clock.tick()
        time_elapsed_since_start += delta_time



import pygame

class Player(pygame.sprite.Sprite):
    def __init__(self):
        super().__init__()
        player_surface = pygame.image.load('data/enemy2.png').convert_alpha();
        player_surface = pygame.transform.scale(player_surface, (64, 64))
        self.image = player_surface
        self.rect = self.image.get_rect(center=(250, 250))
        self.speed = 2

    def player_input_move(self):
        keys = pygame.key.get_pressed()
        if keys[pygame.K_LEFT]:
            self.rect.x -= self.speed
        if keys[pygame.K_RIGHT]:
            self.rect.x += self.speed
        if keys[pygame.K_UP]:
            self.rect.y -= self.speed
        if keys[pygame.K_DOWN]:
            self.rect.y += self.speed

    def update(self):
        self.player_input_move()

#Init
pygame.init()
screen = pygame.display.set_mode((500, 500))
pygame.display.set_caption("Demo")
clock = pygame.time.Clock()
game_active = True

player_group = pygame.sprite.GroupSingle()
player_group.add(Player())

while True:
    for event in pygame.event.get():
        if event.type == pygame.QUIT:
            pygame.quit()
            exit()

    screen.fill((0, 0, 0))

    if game_active:
        player_group.draw(screen)
        player_group.update()

    pygame.display.update()

#Homework:
'''
Find all the images and backgrounds you want to create
For example:
-Player
-Enemies
-Platform
-Ground
-Background
-Dead effect (explosion? Blood spill?, flashes?)
-Sounds effects (shooting?, jumping?, enemy growling?)

Code:
- Create classes and have each load their corresponding sprite
- Draw the onto the screen. 
'''


#Variables
# name = "Kenny"
# number = 10
# print(number + 20)

# number = 20
# if number < 10:
#     print("Number is larger than 10")
# elif 10 <= number <= 19:
#     print("Number is between 10 and 19")
# else:
#     print("Number is less than 10")

# for i in range(5, 12, 2):
#     print(i)

# while True:
#     print("Forever")

# numbers = [1, 3, 5, 7]
# floats = [2.5, 1.2, 9.999, "Kenny"]

# for however in range(len(floats)):
#     print(floats[however])

# for i in floats:
#     print(i)

#Function/Methods
# def Add(num1, num2):
#     return num1 + num2
# Add(5, 7)
#
# def GiveMeFive():
#     print(5)
# GiveMeFive()

# class BankAccount():
#     def __init__(self):
#         self.money = 0
#         self.type = "Chequing"
#
#     def deposit(self, amount):
#         self.money += amount


# bank_account = BankAccount()
# bank_account2 = BankAccount()
# bank_account2.money = 2000
# bank_account.deposit(1000)
# print(bank_account.money)
# print(bank_account2.money)







import pygame

#Game Setup
pygame.init()
screen = pygame.display.set_mode((600, 400))
pygame.display.set_caption('Demo')
clock = pygame.time.Clock()
# test_font = pygame.font.Font('font/XXXXX.ttf', 50)
game_active = True

#Player Class
class Player(pygame.sprite.Sprite):

    def __init__(self):
        super().__init__()
        player_image_frame1 = pygame.image.load('data/alien3.png').convert_alpha()
        player_image_frames = [player_image_frame1]
        self.image = player_image_frames[0]
        self.rect = self.image.get_rect(center = (80, 200))
        self.speed = 4

    def player_input(self):
        keys = pygame.key.get_pressed()
        if keys[pygame.K_LEFT]:
            self.rect.x -= self.speed
        if keys[pygame.K_RIGHT]:
            self.rect.x += self.speed
        if keys[pygame.K_UP]:
            self.rect.y -= self.speed
        if keys[pygame.K_DOWN]:
            self.rect.y += self.speed

    def update(self):
        self.player_input()


#Setup game
sky = pygame.image.load('data/sky.png').convert()
ground = pygame.image.load('data/ground.png').convert()
player = pygame.sprite.GroupSingle()
player.add(Player())


#Main game loop
while True:
    for event in pygame.event.get():
        if event.type == pygame.QUIT:
            pygame.quit()
            exit()

    #Draw background that exists in all mode
    screen.blit(sky, (0, 0))
    screen.blit(ground, (0, 300))

    if game_active:
        player.draw(screen)
        player.update()

    pygame.display.update()
    clock.tick(60)



import os
from setuptools import setup

# Utility function to read the README file.
# Used for the long_description.  It's nice, because now 1) we have a top level
# README file and 2) it's easier to type in the README file than to put a raw
# string in below ...
def read(fname):
    return open(os.path.join(os.path.dirname(__file__), fname)).read()

setup(
    name = "kennytestupload",
    version = "0.0.8",
    author = "Kenny C",
    author_email = "kennycheung388@yahoo.com.hk",
    description = ("An demonstration of how to upload project to pypi."),
    license = "MIT License",
    keywords = "tutorial",
    url = "",
    packages=['kennytestupload'],
    package_data={'kennytestupload':['*']},
    long_description=read('README'),
    classifiers=[
        "Development Status :: 3 - Alpha",
        "License :: OSI Approved :: MIT License",
    ],
    # install_requires=[
    #     'pygame'
    # ],
)



import random
import os

# import basic pygame modules
import pygame as pg

# see if we can load more than standard BMP
if not pg.image.get_extended():
    raise SystemExit("Sorry, extended image module required")


# game constants
MAX_SHOTS = 2  # most player bullets onscreen
ALIEN_ODDS = 22  # chances a new alien appears
BOMB_ODDS = 60  # chances a new bomb will drop
ALIEN_RELOAD = 12  # frames between new aliens
SCREENRECT = pg.Rect(0, 0, 640, 480)
SCORE = 0

main_dir = os.path.split(os.path.abspath(__file__))[0]


def load_image(file):
    """loads an image, prepares it for play"""
    file = os.path.join(main_dir, "data", file)
    try:
        surface = pg.image.load(file)
    except pg.error:
        raise SystemExit(f'Could not load image "{file}" {pg.get_error()}')
    return surface.convert()


def load_sound(file):
    """because pygame can be be compiled without mixer."""
    if not pg.mixer:
        return None
    file = os.path.join(main_dir, "data", file)
    try:
        sound = pg.mixer.Sound(file)
        return sound
    except pg.error:
        print(f"Warning, unable to load, {file}")
    return None


# Each type of game object gets an init and an update function.
# The update function is called once per frame, and it is when each object should
# change its current position and state.
#
# The Player object actually gets a "move" function instead of update,
# since it is passed extra information about the keyboard.


class Player(pg.sprite.Sprite):
    """Representing the player as a moon buggy type car."""

    speed = 10
    bounce = 24
    gun_offset = -11
    images = []

    def __init__(self):
        pg.sprite.Sprite.__init__(self, self.containers)
        self.image = self.images[0]
        self.rect = self.image.get_rect(midbottom=SCREENRECT.midbottom)
        self.reloading = 0
        self.origtop = self.rect.top
        self.facing = -1

    def move(self, direction):
        if direction:
            self.facing = direction
        self.rect.move_ip(direction * self.speed, 0)
        self.rect = self.rect.clamp(SCREENRECT)
        if direction < 0:
            self.image = self.images[0]
        elif direction > 0:
            self.image = self.images[1]
        self.rect.top = self.origtop - (self.rect.left // self.bounce % 2)

    def gunpos(self):
        pos = self.facing * self.gun_offset + self.rect.centerx
        return pos, self.rect.top


class Alien(pg.sprite.Sprite):
    """An alien space ship. That slowly moves down the screen."""

    speed = 13
    animcycle = 12
    images = []

    def __init__(self):
        pg.sprite.Sprite.__init__(self, self.containers)
        self.image = self.images[0]
        self.rect = self.image.get_rect()
        self.facing = random.choice((-1, 1)) * Alien.speed
        self.frame = 0
        if self.facing < 0:
            self.rect.right = SCREENRECT.right

    def update(self):
        self.rect.move_ip(self.facing, 0)
        if not SCREENRECT.contains(self.rect):
            self.facing = -self.facing
            self.rect.top = self.rect.bottom + 1
            self.rect = self.rect.clamp(SCREENRECT)
        self.frame = self.frame + 1
        self.image = self.images[self.frame // self.animcycle % 3]


class Explosion(pg.sprite.Sprite):
    """An explosion. Hopefully the Alien and not the player!"""

    defaultlife = 12
    animcycle = 3
    images = []

    def __init__(self, actor):
        pg.sprite.Sprite.__init__(self, self.containers)
        self.image = self.images[0]
        self.rect = self.image.get_rect(center=actor.rect.center)
        self.life = self.defaultlife

    def update(self):
        """called every time around the game loop.

        Show the explosion surface for 'defaultlife'.
        Every game tick(update), we decrease the 'life'.

        Also we animate the explosion.
        """
        self.life = self.life - 1
        self.image = self.images[self.life // self.animcycle % 2]
        if self.life <= 0:
            self.kill()


class Shot(pg.sprite.Sprite):
    """a bullet the Player sprite fires."""

    speed = -11
    images = []

    def __init__(self, pos):
        pg.sprite.Sprite.__init__(self, self.containers)
        self.image = self.images[0]
        self.rect = self.image.get_rect(midbottom=pos)

    def update(self):
        """called every time around the game loop.

        Every tick we move the shot upwards.
        """
        self.rect.move_ip(0, self.speed)
        if self.rect.top <= 0:
            self.kill()


class Bomb(pg.sprite.Sprite):
    """A bomb the aliens drop."""

    speed = 9
    images = []

    def __init__(self, alien):
        pg.sprite.Sprite.__init__(self, self.containers)
        self.image = self.images[0]
        self.rect = self.image.get_rect(midbottom=alien.rect.move(0, 5).midbottom)

    def update(self):
        """called every time around the game loop.

        Every frame we move the sprite 'rect' down.
        When it reaches the bottom we:

        - make an explosion.
        - remove the Bomb.
        """
        self.rect.move_ip(0, self.speed)
        if self.rect.bottom >= 470:
            Explosion(self)
            self.kill()


class Score(pg.sprite.Sprite):
    """to keep track of the score."""

    def __init__(self):
        pg.sprite.Sprite.__init__(self)
        self.font = pg.font.Font(None, 20)
        self.font.set_italic(1)
        self.color = "white"
        self.lastscore = -1
        self.update()
        self.rect = self.image.get_rect().move(10, 450)

    def update(self):
        """We only update the score in update() when it has changed."""
        if SCORE != self.lastscore:
            self.lastscore = SCORE
            msg = "Score: %d" % SCORE
            self.image = self.font.render(msg, 0, self.color)


def main(winstyle=0):
    # Initialize pygame
    if pg.get_sdl_version()[0] == 2:
        pg.mixer.pre_init(44100, 32, 2, 1024)
    pg.init()
    if pg.mixer and not pg.mixer.get_init():
        print("Warning, no sound")
        pg.mixer = None

    fullscreen = False
    # Set the display mode
    winstyle = 0  # |FULLSCREEN
    bestdepth = pg.display.mode_ok(SCREENRECT.size, winstyle, 32)
    screen = pg.display.set_mode(SCREENRECT.size, winstyle, bestdepth)

    # Load images, assign to sprite classes
    # (do this before the classes are used, after screen setup)
    img = load_image("player1.gif")
    Player.images = [img, pg.transform.flip(img, 1, 0)]
    img = load_image("explosion1.gif")
    Explosion.images = [img, pg.transform.flip(img, 1, 1)]
    Alien.images = [load_image(im) for im in ("alien1.gif", "alien2.gif", "alien3.gif")]
    Bomb.images = [load_image("bomb.gif")]
    Shot.images = [load_image("shot.gif")]

    # decorate the game window
    icon = pg.transform.scale(Alien.images[0], (32, 32))
    pg.display.set_icon(icon)
    pg.display.set_caption("Pygame Aliens")
    pg.mouse.set_visible(0)

    # create the background, tile the bgd image
    bgdtile = load_image("background.gif")
    background = pg.Surface(SCREENRECT.size)
    for x in range(0, SCREENRECT.width, bgdtile.get_width()):
        background.blit(bgdtile, (x, 0))
    screen.blit(background, (0, 0))
    pg.display.flip()

    # load the sound effects
    boom_sound = load_sound("boom.wav")
    shoot_sound = load_sound("car_door.wav")
    if pg.mixer:
        music = os.path.join(main_dir, "data", "house_lo.wav")
        pg.mixer.music.load(music)
        pg.mixer.music.play(-1)

    # Initialize Game Groups
    aliens = pg.sprite.Group()
    shots = pg.sprite.Group()
    bombs = pg.sprite.Group()
    all = pg.sprite.RenderUpdates()
    lastalien = pg.sprite.GroupSingle()

    # assign default groups to each sprite class
    Player.containers = all
    Alien.containers = aliens, all, lastalien
    Shot.containers = shots, all
    Bomb.containers = bombs, all
    Explosion.containers = all
    Score.containers = all

    # Create Some Starting Values
    global score
    alienreload = ALIEN_RELOAD
    clock = pg.time.Clock()

    # initialize our starting sprites
    global SCORE
    player = Player()
    Alien()  # note, this 'lives' because it goes into a sprite group
    if pg.font:
        all.add(Score())

    # Run our main loop whilst the player is alive.
    while player.alive():
        # get input
        for event in pg.event.get():
            if event.type == pg.QUIT:
                return
            if event.type == pg.KEYDOWN and event.key == pg.K_ESCAPE:
                return
            elif event.type == pg.KEYDOWN:
                if event.key == pg.K_f:
                    if not fullscreen:
                        print("Changing to FULLSCREEN")
                        screen_backup = screen.copy()
                        screen = pg.display.set_mode(
                            SCREENRECT.size, winstyle | pg.FULLSCREEN, bestdepth
                        )
                        screen.blit(screen_backup, (0, 0))
                    else:
                        print("Changing to windowed mode")
                        screen_backup = screen.copy()
                        screen = pg.display.set_mode(
                            SCREENRECT.size, winstyle, bestdepth
                        )
                        screen.blit(screen_backup, (0, 0))
                    pg.display.flip()
                    fullscreen = not fullscreen

        keystate = pg.key.get_pressed()

        # clear/erase the last drawn sprites
        all.clear(screen, background)

        # update all the sprites
        all.update()

        # handle player input
        direction = keystate[pg.K_RIGHT] - keystate[pg.K_LEFT]
        player.move(direction)
        firing = keystate[pg.K_SPACE]
        if not player.reloading and firing and len(shots) < MAX_SHOTS:
            Shot(player.gunpos())
            if pg.mixer:
                shoot_sound.play()
        player.reloading = firing

        # Create new alien
        if alienreload:
            alienreload = alienreload - 1
        elif not int(random.random() * ALIEN_ODDS):
            Alien()
            alienreload = ALIEN_RELOAD

        # Drop bombs
        if lastalien and not int(random.random() * BOMB_ODDS):
            Bomb(lastalien.sprite)

        # Detect collisions between aliens and players.
        for alien in pg.sprite.spritecollide(player, aliens, 1):
            if pg.mixer:
                boom_sound.play()
            Explosion(alien)
            Explosion(player)
            SCORE = SCORE + 1
            player.kill()

        # See if shots hit the aliens.
        for alien in pg.sprite.groupcollide(aliens, shots, 1, 1).keys():
            if pg.mixer:
                boom_sound.play()
            Explosion(alien)
            SCORE = SCORE + 1

        # See if alien bombs hit the player.
        for bomb in pg.sprite.spritecollide(player, bombs, 1):
            if pg.mixer:
                boom_sound.play()
            Explosion(player)
            Explosion(bomb)
            player.kill()

        # draw the scene
        dirty = all.draw(screen)
        pg.display.update(dirty)

        # cap the framerate at 40fps. Also called 40HZ or 40 times per second.
        clock.tick(40)

    if pg.mixer:
        pg.mixer.music.fadeout(1000)
    pg.time.wait(1000)


# call the "main" function if running this script
if __name__ == "__main__":
    main()
    pg.quit()
